package de.intevation.flys.artifacts.states;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import de.intevation.artifactdatabase.state.Facet;
import de.intevation.artifactdatabase.data.StateData;

import de.intevation.artifacts.CallContext;
import de.intevation.artifacts.Artifact;
import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.WINFOArtifact;

import de.intevation.flys.artifacts.math.WKmsOperation;

import de.intevation.flys.artifacts.model.CalculationResult;
import de.intevation.flys.artifacts.model.DataFacet;
import de.intevation.flys.artifacts.model.DifferenceCurveFacet;
import de.intevation.flys.artifacts.model.FacetTypes;
import de.intevation.flys.artifacts.model.WaterlevelFacet;
import de.intevation.flys.artifacts.model.WKms;
import de.intevation.flys.artifacts.model.WQKms;

import de.intevation.flys.utils.FLYSUtils;


public class WDifferencesState
extends      DefaultState
implements   FacetTypes
{
    /** The logger that is used in this state. */
    private static Logger logger = Logger.getLogger(WDifferencesState.class);


    public WDifferencesState() {
    }


    /** Specify to display nothing (this is kind of a "final" state). */
    @Override
    protected String getUIProvider() {
        return "noinput";
    }


    @Override
    public boolean validate(Artifact artifact)
    throws IllegalArgumentException
    {
        FLYSArtifact flys = (FLYSArtifact) artifact;

        StateData data = flys.getData("diffids");

        if (data == null) {
            throw new IllegalArgumentException("diffids is empty");
        }

        // TODO: Also validate format.

        return true;
    }


    /**
     * Return List of WKms that are difference of pairs.
     */
    @Override
    public Object computeAdvance(
        FLYSArtifact artifact,
        String       hash,
        CallContext  context,
        List<Facet>  facets,
        Object       old
    ) {
        WINFOArtifact winfo = (WINFOArtifact)artifact;
        String id = getID();

        // Load the Artifacts/facets that we want to subtract and display.
        // Expected format is:
        // [42537f1e-3522-42ef-8968-635b03d8e9c6;longitudinal_section.w;0]#[1231f2-....]
        String datas[] = winfo.getDataAsString("diffids").split("#");

        // Validate the Data-Strings.
        for (String s: datas) {
            if (!WaterlevelSelectState.isValueValid(winfo.getDataAsString("diffids"))) {
                // TODO: escalate.
            }
        }

        if (datas.length < 2) {
            // TODO crash with style
            ;
        }

        List<WKms> wkmss = new ArrayList<WKms>();

        for(int i = 0; i < datas.length /2; i++) {
            String uuid1 = WaterlevelSelectState.strip(datas[i+0]).split(";")[0];
            String uuid2 = WaterlevelSelectState.strip(datas[i+1]).split(";")[0];
    
            WINFOArtifact flys1 = (WINFOArtifact) FLYSUtils.getArtifact(
                uuid1,
                context);
            WINFOArtifact flys2 = (WINFOArtifact) FLYSUtils.getArtifact(
                uuid2,
                context);
    
            if (flys1 == null) {
                logger.warn("One of the artifacts (1) for diff calculation could not be loaded");
            }
            if (flys2 == null) {
                logger.warn("One of the artifacts (2) for diff calculation could not be loaded");
            }
            // TODO: Issue of multiple results in calculation
            String facetName = "diff ()";
    
            if (flys1 != null && flys2 != null) {
                // TODO Also check size.
                // TODO Also need index of wqkms.
                WQKms[] minuend    = (WQKms[]) flys1.getWaterlevelData().
                                                     getData();
                WQKms[] subtrahend = (WQKms[]) flys2.getWaterlevelData().
                                                     getData();

                if (datas.length > 2) {
                    logger.error("Cannot keep more than one calculation.");
                }
                facetName = "W (" + minuend[0].getName() + ") - W (" + subtrahend[0].getName() + ")";
                WKms wkms = WKmsOperation.SUBTRACTION.operate(minuend[0], subtrahend[0]);
                wkms.setName(facetName);
                wkmss.add(wkms);
                logger.debug("WKMSSubtraction happened");
            }
    
            if (facets != null) {
                // TODO: pass computetype and state id.
                //, ComputeType.ADVANCE, getID(), hash));
                //facets.add(new DataFacet(W_DIFFERENCES, facetName));
                facets.add(new DifferenceCurveFacet(i, W_DIFFERENCES, facetName,
                    ComputeType.ADVANCE, id, hash));
                //facets.add(new WaterlevelFacet
                facets.add(new DataFacet(CSV, "CSV data"));
            }
            else {
                logger.debug("Not adding facets in WDifferencesState.");
            }
        }

        CalculationResult result = new CalculationResult();
        WKms[] diffs = wkmss.toArray(new WKms[wkmss.size()]);
        result.setData(diffs);
        return result;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
